                     /*-----------------------------------*/
                     /*             >>>Slip<<<            */
                     /*            Theo D'Hondt           */
                     /*     VUB Software Languages Lab    */
                     /*             (c) 2010              */
                     /*-----------------------------------*/
                     /*version 3: factored out environment*/
                     /*-----------------------------------*/
                     /*              natives              */
                     /*-----------------------------------*/

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "SlipSlip.h"

#include "SlipMain.h"

#include "SlipDictionary.h"
#include "SlipEvaluate.h"
#include "SlipNative.h"
#include "SlipPool.h"
#include "SlipPrint.h"
#include "SlipRead.h"

/*------------------------------ private macros ------------------------------*/

#define chunk_size(TYP)                                                        \
  ((sizeof(TYP) - sizeof(CNT))/ sizeof(CEL_type))

/*------------------------------ private constants ---------------------------*/

static const TXT_type AL1_error_string = "at least one argument required";
static const TXT_type AL2_error_string = "at least two arguments required";
static const TXT_type AM1_error_string = "at most one argument required";
static const TXT_type AM2_error_string = "at most two arguments required";
static const TXT_type ARN_error_string = "arguments restricted to numbers";
static const TXT_type EX0_error_string = "no arguments allowed";
static const TXT_type EX1_error_string = "exactly one argument required";
static const TXT_type EX2_error_string = "exactly two arguments required";
static const TXT_type EX3_error_string = "exactly three arguments required";
static const TXT_type IOR_error_string = "index out of range";
static const TXT_type NAL_error_string = "not a list";
static const TXT_type NAN_error_string = "not a number";
static const TXT_type NAP_error_string = "not a pair";
static const TXT_type NAS_error_string = "not a string";
static const TXT_type NAV_error_string = "not a vector";
static const TXT_type NNN_error_string = "non-negative number required";

/*------------------------------ private functions ---------------------------*/

static NIL_type native_define(TXT_type String,
                              FUN_type Function)
  { EXP_type native;
    SYM_type variable;
    variable = Pool_Enter(String);
    native = make_NAT(Function,
                      String);
    Dictionary_Define(variable,
                      native); }
                                               
/*----------------------------------------------------------------------------*/
/*------------------------------ variables -----------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ meta-circularity-level ----------------------*/

static const TXT_type mcl_string = "circularity-level";

static NIL_type initialize_circularity_level(NIL_type)
  { SYM_type variable;
    variable = Pool_Enter(mcl_string);
    Dictionary_Define(variable,
                      Main_Zero); }

/*----------------------------------------------------------------------------*/
/*------------------------------ arithmetic ----------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ + -------------------------------------------*/

static const TXT_type add_string = "+";

static EXP_type add_floats(FLO_type Raw_real,
                           VEC_type Vector,
                           UNS_type Index)
  { EXP_type value;
    FLO_type raw_real,
             raw_real_sum;
    LNG_type raw_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_real_sum = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              raw_real_sum += raw_number;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              raw_real_sum += raw_real;
              break;
            default:
              return Main_Error_Text(ARN_error_string,
                                     add_string); }}
    return make_REA(raw_real_sum); }

static EXP_type add_integers(NBR_type Number,
                             VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real,
             raw_real_sum;
    LNG_type raw_number,
             raw_number_sum;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_number_sum = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              raw_number_sum += raw_number;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              raw_real_sum = raw_number_sum + raw_real;
              return add_floats(raw_real_sum,
                                Vector,
                                index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     add_string); }}
    return make_NBR(raw_number_sum); }

static NIL_type add_native(VEC_type Vector,
                           CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size == 0)
      value = Main_Zero;
    else
      { value = Vector[1];
        if (size > 1)
          { tag = Tag_of(value);
            switch (tag)
              { case NBR_tag:
                  value = add_integers(value,
                                       Vector);
                  break;
                case REA_tag:
                  real = value;
                  raw_real = real->flo;
                  value = add_floats(raw_real,
                                     Vector,
                                     2); 
                  break;
                default:
                  Main_Error_Text(ARN_error_string,
                                  add_string); }}}
    Evaluate_Continue(value,
                      Continuation); }
                      
static NIL_type initialize_add_native(NIL_type)
  { native_define(add_string,
                  add_native); }

/*------------------------------ - -------------------------------------------*/

static const TXT_type sub_string = "-";

static EXP_type subtract_floats(FLO_type Raw_real,
                                VEC_type Vector,
                                UNS_type Index)
  { EXP_type value;
    FLO_type raw_real,
             raw_real_sum;
    LNG_type raw_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_real_sum = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              raw_real_sum -= raw_number;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              raw_real_sum -= raw_real;
              break;
            default:
              return Main_Error_Text(ARN_error_string,
                                     sub_string); }}
    return make_REA(raw_real_sum); }

static EXP_type subtract_integers(NBR_type Number,
                                  VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real,
             raw_real_sum;
    LNG_type raw_number,
             raw_number_sum;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_number_sum = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              raw_number_sum -= raw_number;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              raw_real_sum = raw_number_sum - raw_real;
              return subtract_floats(raw_real_sum,
                                     Vector,
                                     index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     sub_string); }}
    return make_NBR(raw_number_sum); }

static NIL_type subtract_native(VEC_type Vector,
                                CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    LNG_type raw_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size == 0)
      Main_Error_Text(AL1_error_string,
                      sub_string);
    value = Vector[1];
    tag = Tag_of(value);
    if (size == 1)
      switch (tag)
        { case NBR_tag:
            number = value;
            raw_number = number->lng;
            value = make_NBR(- raw_number);
            break;
          case REA_tag:
            real = value;
            raw_real = real->flo;
            value = make_REA(- raw_real); 
            break;
          default:
            Main_Error_Text(ARN_error_string,
                            sub_string); }
    else
      switch (tag)
        { case NBR_tag:
            value = subtract_integers(value,
                                      Vector);
            break;
          case REA_tag:
            real = value;
            raw_real = real->flo;
            value = subtract_floats(raw_real,
                                    Vector,
                                    2); 
            break;
          default:
            Main_Error_Text(ARN_error_string,
                            sub_string); }
    Evaluate_Continue(value,
                      Continuation); }
                      
static NIL_type initialize_subtract_native(NIL_type)
  { native_define(sub_string,
                  subtract_native); }

/*------------------------------ * -------------------------------------------*/

static const TXT_type mul_string = "*";

static EXP_type multiply_floats(FLO_type Raw_real,
                                VEC_type Vector,
                                UNS_type Index)
  { EXP_type value;
    FLO_type raw_real,
             raw_real_product;
    LNG_type raw_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_real_product = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag   = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              raw_real_product *= raw_number;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              raw_real_product *= raw_real;
              break;
            default:
              return Main_Error_Text(ARN_error_string,
                                     mul_string); }}
    return make_REA(raw_real_product); }

static EXP_type multiply_integers(NBR_type Number,
                                  VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real,
             raw_real_product;
    LNG_type raw_number,
             raw_number_product;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_number_product = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value  = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              raw_number_product *= raw_number;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              raw_real_product = raw_number_product * raw_real;
              return multiply_floats(raw_real_product,
                                     Vector,
                                     index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     mul_string); }}
    return make_NBR(raw_number_product); }

static NIL_type multiply_native(VEC_type Vector,
                                CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size == 0)
      value = Main_One;
    else 
      { value = Vector[1];
        if (size > 1)
          { tag = Tag_of(value);
            switch (tag)
              { case NBR_tag:
                  value = multiply_integers(value,
                                            Vector);
                  break;
                case REA_tag:
                  real = value;
                  raw_real = real->flo;
                  value = multiply_floats(raw_real,
                                         Vector,
                                         2); 
                  break;
                default:
                  Main_Error_Text(ARN_error_string,
                                  mul_string); }}}
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_multiply_native(NIL_type)
  { native_define(mul_string,
                  multiply_native); }

/*------------------------------ / -------------------------------------------*/

static const TXT_type div_string = "/";

static EXP_type divide_floats(FLO_type Raw_real,
                              VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real,
             raw_real_product;
    LNG_type raw_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_real_product = Raw_real;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              raw_real_product /= raw_number;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              raw_real_product /= raw_real;
              break;
            default:
              return Main_Error_Text(ARN_error_string,
                                     div_string); }}
    return make_REA(raw_real_product); }

static NIL_type divide_native(VEC_type Vector,
                              CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    LNG_type raw_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size == 0)
      Main_Error_Text(AL1_error_string,
                      div_string);
    value = Vector[1];
    tag = Tag_of(value);
    if (size == 1)
      switch (tag)
        { case NBR_tag:
            number = value;
            raw_number = number->lng;
            value = make_REA(1.0 / raw_number);
            break;
          case REA_tag:
            real = value;
            raw_real = real->flo;
            value =  make_REA(1.0 / raw_real);
            break;
          default:
            Main_Error_Text(ARN_error_string,
                            div_string); }
    else
      switch (tag)
        { case NBR_tag:
            number = value;
            raw_number = number->lng;
            value = divide_floats(raw_number,
                                  Vector);
            break;
          case REA_tag:
            real = value;
            raw_real = real->flo;
            value = divide_floats(raw_real,
                                  Vector);
            break;
          default:
            Main_Error_Text(ARN_error_string,
                            div_string); }
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_divide_native(NIL_type)
  { native_define(div_string,
                  divide_native); }
                  
/*------------------------------ quotient ------------------------------------*/

static const TXT_type quo_string = "quotient";

static NIL_type quotient_native(VEC_type Vector,
                                CNT_type Continuation)
  { LNG_type raw_left,
             raw_number,
             raw_right;
    NBR_type left,
             number,
             right;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      quo_string);
    left = Vector[1];
    right = Vector[2];
    if (!is_NBR(left) || !is_NBR(right))
      Main_Error_Text(ARN_error_string,
                      quo_string);
    raw_left = left->lng;
    raw_right = right->lng;
    raw_number = raw_left / raw_right;
    number = make_NBR(raw_number);
    Evaluate_Continue(number,
                      Continuation); }

static NIL_type initialize_quotient_native(NIL_type)
  { native_define(quo_string,
                  quotient_native); }

/*------------------------------ remainder -----------------------------------*/

static const TXT_type rem_string = "remainder";

static NIL_type remainder_native(VEC_type Vector,
                                 CNT_type Continuation)
  { LNG_type raw_left,
             raw_number,
             raw_right;
    NBR_type left,
             number,
             right;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      rem_string);
    left = Vector[1];
    right = Vector[2];
    if (!is_NBR(left) || !is_NBR(right))
      Main_Error_Text(ARN_error_string,
                      rem_string);
    raw_left = left->lng;
    raw_right = right->lng;
    raw_number = raw_left % raw_right;
    number = make_NBR(raw_number);
    Evaluate_Continue(number,
                      Continuation); }

static NIL_type initialize_remainder_native(NIL_type)
  { native_define(rem_string,
                  remainder_native); }

/*----------------------------------------------------------------------------*/
/*----------------------- transcendent functions -----------------------------*/
/*----------------------------------------------------------------------------*/

/*--------------------------------- sqrt -------------------------------------*/

static const TXT_type sqt_string = "sqrt";

static NIL_type sqrt_native(VEC_type Vector,
                            CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_float;
    LNG_type raw_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                      sqt_string);
    value = Vector[1];
    tag = Tag_of(value);
    switch (tag)
      { case NBR_tag:
          number = value;
          raw_number = number->lng;
          raw_float = sqrt(raw_number);
          break;
        case REA_tag:
          real = value;
          raw_float = real->flo;
          raw_float = sqrt(raw_float);
          break;
        default:
          Main_Error_Text(ARN_error_string,
                          sqt_string); }
    real = make_REA(raw_float);
    Evaluate_Continue(real,
                      Continuation); }
        
static NIL_type initialize_sqrt_native(NIL_type)
  { native_define(sqt_string,
                  sqrt_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------ comparison ----------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ = -------------------------------------------*/

static const TXT_type eql_string = "=";

static EXP_type equal_floats(FLO_type Raw_real,
                             VEC_type Vector,
                             UNS_type Index)
  { EXP_type value;
    FLO_type raw_previous_real,
             raw_real;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_real = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_real = number->lng;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              break;
            default:
              Main_Error_Text(ARN_error_string,
                              eql_string); }
        if (raw_previous_real != raw_real)
          return Main_False;
        raw_previous_real = raw_real; }
   return Main_True; }

static EXP_type equal_integers(NBR_type Number,
                               VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real;
    LNG_type raw_number,
             raw_previous_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_number = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              if (raw_previous_number != raw_number)
                return Main_False;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              if (raw_previous_number != raw_real)
                return Main_False;
              return equal_floats(raw_real,
                                  Vector,
                                  index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     eql_string); }
        raw_previous_number = raw_number; }
    return Main_True; }

static NIL_type equal_native(VEC_type Vector,
                             CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size < 2)
      Main_Error_Text(AL2_error_string,
                      eql_string);
    value = Vector[1];
    tag = Tag_of(value);
    switch (tag)
      { case NBR_tag:
          value = equal_integers(value,
                                 Vector);
          break;
        case REA_tag:
          real = value;
          raw_real = real->flo;
          value = equal_floats(raw_real,
                               Vector,
                               2);
          break;
        default:
          Main_Error_Text(ARN_error_string,
                          eql_string); }
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_equal_native(NIL_type)
  { native_define(eql_string,
                  equal_native); }

/*------------------------------ > -------------------------------------------*/

static const TXT_type grt_string = ">";

static EXP_type greater_floats(FLO_type Raw_real,
                               VEC_type Vector,
                               UNS_type Index)
  { EXP_type value;
    FLO_type raw_previous_real,
             raw_real;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_real = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_real = number->lng;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              break;
            default:
              Main_Error_Text(ARN_error_string,
                              grt_string); }
        if (raw_previous_real <= raw_real)
          return Main_False;
        raw_previous_real = raw_real; }
   return Main_True; }

static EXP_type greater_integers(NBR_type Number,
                               VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real;
    LNG_type raw_number,
             raw_previous_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_number = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              if (raw_previous_number <= raw_number)
                return Main_False;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              if (raw_previous_number <= raw_real)
                return Main_False;
              return equal_floats(raw_real,
                                  Vector,
                                  index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     grt_string); }
        raw_previous_number = raw_number; }
    return Main_True; }

static NIL_type greater_native(VEC_type Vector,
                               CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size < 2)
      Main_Error_Text(AL2_error_string,
                      grt_string);
    value = Vector[1];
    tag = Tag_of(value);
    switch (tag)
      { case NBR_tag:
          value = greater_integers(value,
                                   Vector);
          break;
        case REA_tag:
          real = value;
          raw_real = real->flo;
          value = greater_floats(raw_real,
                                 Vector,
                                 2);
          break;
        default:
          Main_Error_Text(ARN_error_string,
                          grt_string); }
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_greater_native(NIL_type)
  { native_define(grt_string,
                  greater_native); }

/*------------------------------ < -------------------------------------------*/

static const TXT_type lss_string = "<";

static EXP_type less_floats(FLO_type Raw_real,
                            VEC_type Vector,
                            UNS_type Index)
  { EXP_type value;
    FLO_type raw_previous_real,
             raw_real;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_real = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_real = number->lng;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              break;
            default:
              Main_Error_Text(ARN_error_string,
                              lss_string); }
        if (raw_previous_real >= raw_real)
          return Main_False;
        raw_previous_real = raw_real; }
   return Main_True; }

static EXP_type less_integers(NBR_type Number,
                              VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real;
    LNG_type raw_number,
             raw_previous_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_number = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              if (raw_previous_number >= raw_number)
                return Main_False;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              if (raw_previous_number >= raw_real)
                return Main_False;
              return equal_floats(raw_real,
                                  Vector,
                                  index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     lss_string); }
        raw_previous_number = raw_number; }
    return Main_True; }

static NIL_type less_native(VEC_type Vector,
                            CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size < 2)
      Main_Error_Text(AL2_error_string,
                      lss_string);
    value = Vector[1];
    tag = Tag_of(value);
    switch (tag)
      { case NBR_tag:
          value = less_integers(value,
                                Vector);
          break;
        case REA_tag:
          real = value;
          raw_real = real->flo;
          value = less_floats(raw_real,
                              Vector,
                              2);
          break;
        default:
          Main_Error_Text(ARN_error_string,
                          lss_string); }
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_less_native(NIL_type)
  { native_define(lss_string,
                  less_native); }

/*------------------------------ >= ------------------------------------------*/

static const TXT_type geq_string = ">=";

static EXP_type greater_or_equal_floats(FLO_type Raw_real,
                                        VEC_type Vector,
                                        UNS_type Index)
  { EXP_type value;
    FLO_type raw_previous_real,
             raw_real;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_real = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_real = number->lng;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              break;
            default:
              Main_Error_Text(ARN_error_string,
                              geq_string); }
        if (raw_previous_real < raw_real)
          return Main_False;
        raw_previous_real = raw_real; }
   return Main_True; }

static EXP_type greater_or_equal_integers(NBR_type Number,
                                          VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real;
    LNG_type raw_number,
             raw_previous_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_number = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              if (raw_previous_number < raw_number)
                return Main_False;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              if (raw_previous_number < raw_real)
                return Main_False;
              return equal_floats(raw_real,
                                  Vector,
                                  index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     geq_string); }
        raw_previous_number = raw_number; }
    return Main_True; }

static NIL_type greater_or_equal_native(VEC_type Vector,
                                        CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size < 2)
      Main_Error_Text(AL2_error_string,
                      geq_string);
    value = Vector[1];
    tag = Tag_of(value);
    switch (tag)
      { case NBR_tag:
          value = greater_or_equal_integers(value,
                                            Vector);
          break;
        case REA_tag:
          real = value;
          raw_real = real->flo;
          value = greater_or_equal_floats(raw_real,
                                          Vector,
                                          2);
          break;
        default:
          Main_Error_Text(ARN_error_string,
                          geq_string); }
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_greater_or_equal_native(NIL_type)
  { native_define(geq_string,
                  greater_or_equal_native); }

/*------------------------------ <= ------------------------------------------*/

static const TXT_type leq_string = "<=";

static EXP_type less_or_equal_floats(FLO_type Raw_real,
                                     VEC_type Vector,
                                     UNS_type Index)
  { EXP_type value;
    FLO_type raw_previous_real,
             raw_real;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_real = Raw_real;
    size = size_VEC(Vector);
    for (index = Index;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_real = number->lng;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              break;
            default:
              Main_Error_Text(ARN_error_string,
                              leq_string); }
        if (raw_previous_real > raw_real)
          return Main_False;
        raw_previous_real = raw_real; }
   return Main_True; }

static EXP_type less_or_equal_integers(NBR_type Number,
                                       VEC_type Vector)
  { EXP_type value;
    FLO_type raw_real;
    LNG_type raw_number,
             raw_previous_number;
    NBR_type number;
    REA_type real;
    TAG_type tag;
    UNS_type index,
             size;
    raw_previous_number = Number->lng;
    size = size_VEC(Vector);
    for (index = 2;
         index <= size;
         index++)
      { value = Vector[index];
        tag = Tag_of(value);
        switch (tag)
          { case NBR_tag:
              number = value;
              raw_number = number->lng;
              if (raw_previous_number > raw_number)
                return Main_False;
              break;
            case REA_tag:
              real = value;
              raw_real = real->flo;
              if (raw_previous_number > raw_real)
                return Main_False;
              return equal_floats(raw_real,
                                  Vector,
                                  index + 1);
            default:
              return Main_Error_Text(ARN_error_string,
                                     leq_string); }
        raw_previous_number = raw_number; }
    return Main_True; }

static NIL_type less_or_equal_native(VEC_type Vector,
                                        CNT_type Continuation)
  { EXP_type value;
    FLO_type raw_real;
    REA_type real;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size < 2)
      Main_Error_Text(AL2_error_string,
                      leq_string);
    value = Vector[1];
    tag = Tag_of(value);
    switch (tag)
      { case NBR_tag:
          value = less_or_equal_integers(value,
                                         Vector);
          break;
        case REA_tag:
          real = value;
          raw_real = real->flo;
          value = less_or_equal_floats(raw_real,
                                       Vector,
                                       2);
          break;
        default:
          Main_Error_Text(ARN_error_string,
                          leq_string); }
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_less_or_equal_native(NIL_type)
  { native_define(leq_string,
                  less_or_equal_native); }

/*------------------------------ eq? -----------------------------------------*/

static const TXT_type eqq_string = "eq?";

static NIL_type equivalent_native(VEC_type Vector,
                                  CNT_type Continuation)
  { ADR_type left,
             right;
    EXP_type value;
    UNS_type size;
    left  = Vector[1],
    right = Vector[2];
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                              eqq_string);
    if (left == right)
      value = Main_True;
    else
      value = Main_False;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_equivalent_native(NIL_type)
  { native_define(eqq_string,
                  equivalent_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------ booleans ------------------------------------*/
/*----------------------------------------------------------------------------*/

/*-------------------------------- not ---------------------------------------*/

static const TXT_type not_string = "not";

static NIL_type not_native(VEC_type Vector,
                           CNT_type Continuation)
  { EXP_type value;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                      not_string);
    value = Vector[1];
    if (is_FLS(value))
      value = Main_True;
    else
      value = Main_False;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_not_native(NIL_type)
  { native_define(not_string,
                  not_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------ predicates ----------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ null? ---------------------------------------*/

static const TXT_type inl_string = "null?";

static NIL_type is_null_native(VEC_type Vector,
                               CNT_type Continuation)
  { EXP_type expression,
             value;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                              inl_string);
    expression = Vector[1];
    if (is_NUL(expression))
      value = Main_True;
    else
      value = Main_False;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_is_null_native(NIL_type)
  { native_define(inl_string,
                  is_null_native); }

/*------------------------------ pair? -------------------------------------*/

static const TXT_type ipr_string = "pair?";

static NIL_type is_pair_native(VEC_type Vector,
                               CNT_type Continuation)
  { EXP_type expression,
             value;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                              ipr_string);
    expression = Vector[1];
    if (is_PAI(expression))
      value = Main_True;
    else
      value = Main_False;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_is_pair_native(NIL_type)
  { native_define(ipr_string,
                  is_pair_native); }

/*------------------------------ symbol? -------------------------------------*/

static const TXT_type ism_string = "symbol?";

static NIL_type is_symbol_native(VEC_type Vector,
                                 CNT_type Continuation)
  { EXP_type expression,
             value;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                              ism_string);
    expression = Vector[1];
    if (is_SYM(expression))
      value = Main_True;
    else
      value = Main_False;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_is_symbol_native(NIL_type)
  { native_define(ism_string,
                  is_symbol_native); }

/*------------------------------ vector? -------------------------------------*/

static const TXT_type ivc_string = "vector?";

static NIL_type is_vector_native(VEC_type Vector,
                                 CNT_type Continuation)
  { EXP_type expression,
             value;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                              ivc_string);
    expression = Vector[1];
    if (is_VEC(expression))
      value = Main_True;
    else
      value = Main_False;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_is_vector_native(NIL_type)
  { native_define(ivc_string,
                  is_vector_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------ pairs ---------------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ cons ----------------------------------------*/

static const TXT_type cns_string = "cons";

static NIL_type cons_native(VEC_type Vector,
                            CNT_type Continuation)
  { EXP_type car,
             cdr;
    PAI_type pair;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                              cns_string);
    car = Vector[1];
    cdr = Vector[2];
    pair = make_PAI(car,
                    cdr);
    Evaluate_Continue(pair,
                      Continuation); }

static NIL_type initialize_cons_native(NIL_type)
  { native_define(cns_string,
                  cons_native); }

/*------------------------------ car -----------------------------------------*/

static const TXT_type car_string = "car";

static NIL_type car_native(VEC_type Vector,
                           CNT_type Continuation)
  { EXP_type value;
    PAI_type pair;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                      car_string);
    pair = Vector[1];
    if (!is_PAI(pair))
      Main_Error_Text(NAP_error_string,
                      car_string);
    value = pair->car;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_car_native(NIL_type)
  { native_define(car_string,
                  car_native); }

/*------------------------------ cdr -----------------------------------------*/

static const TXT_type cdr_string = "cdr";

static NIL_type cdr_native(VEC_type Vector,
                           CNT_type Continuation)
  { EXP_type value;
    PAI_type pair;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                      cdr_string);
    pair = Vector[1];
    if (!is_PAI(pair))
      Main_Error_Text(NAP_error_string,
                      cdr_string);
    value = pair->cdr;
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_cdr_native(NIL_type)
  { native_define(cdr_string,
                  cdr_native); }

/*------------------------------ set-car! ------------------------------------*/

static const TXT_type sca_string = "set-car!";

static NIL_type set_car_native(VEC_type Vector,
                               CNT_type Continuation)
  { EXP_type value;
    PAI_type pair;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      sca_string);
    pair = Vector[1];
    value = Vector[2];
    if (!is_PAI(pair))
      Main_Error_Text(NAP_error_string,
                      sca_string);
    pair->car = value;
    Evaluate_Continue(pair,
                      Continuation); }

static NIL_type initialize_set_car_native(NIL_type)
  { native_define(sca_string,
                  set_car_native); }

/*------------------------------ set-cdr! ------------------------------------*/

static const TXT_type scd_string = "set-cdr!";

static NIL_type set_cdr_native(VEC_type Vector,
                               CNT_type Continuation)
  { EXP_type value;
    PAI_type pair;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      scd_string);
    pair = Vector[1];
    value = Vector[2];
    if (!is_PAI(pair))
      Main_Error_Text(NAP_error_string,
                      scd_string);
    pair->cdr = value;
    Evaluate_Continue(pair,
                      Continuation); }

static NIL_type initialize_set_cdr_native(NIL_type)
  { native_define(scd_string,
                  set_cdr_native); }

/*------------------------------ assoc ---------------------------------------*/

static const TXT_type ass_string = "assoc";

static NIL_type assoc_native(VEC_type Vector,
                             CNT_type Continuation)
  { EXP_type value;
    PAI_type list,
             pair;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      ass_string);
    value = Vector[1];
    list = Vector[2];
    for (;;)
      { tag = Tag_of(list);
        switch (tag)
          { case NUL_tag:
              Evaluate_Continue(Main_False,
                                Continuation);
            case PAI_tag:
              pair = list->car;
              if (!is_PAI(pair))
                Main_Error_Text(NAP_error_string,
                                ass_string);
              if (value == pair->car)
                Evaluate_Continue(pair,
                                  Continuation);
              list = list->cdr;
              break;
            default:
              Main_Error_Text(NAL_error_string,
                              ass_string); }}}

static NIL_type initialize_assoc_native(NIL_type)
  { native_define(ass_string,
                  assoc_native); }

/*----------------------------------------------------------------------------*/
/*-------------------------------- vectors -----------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ make-vector ---------------------------------*/

static const TXT_type mkv_string = "make-vector";

static NIL_type make_vector_native(VEC_type Vector,
                                   CNT_type Continuation)
  { EXP_type value;
    LNG_type raw_number;
    NBR_type number;
    UNS_type index,
             size;
    VEC_type vector;
    size = size_VEC(Vector);
    if (size == 0)
      Main_Error_Text(AL1_error_string,
                      mkv_string);
    if (size > 2)
      Main_Error_Text(AM2_error_string,
                      mkv_string);
    value = (size == 1)? Main_Unspecified: Vector[2];
    number = Vector[1];
    if (!is_NBR(number))
      Main_Error_Text(NAN_error_string,
                      mkv_string);
    raw_number = number->lng;
    if (raw_number < 0)
      Main_Error_Text(NNN_error_string,
                      mkv_string);
    if (raw_number == 0)
      Evaluate_Continue(Main_Empty_Vector,
                        Continuation);
    vector = make_VEC(raw_number);
    for (index = 1;
         index <= raw_number;
         index++)
      vector[index] = value;
    Evaluate_Continue(vector,
                      Continuation); }

static NIL_type initialize_make_vector_native(NIL_type)
  { native_define(mkv_string,
                  make_vector_native); }

/*------------------------------ vector-ref ---------------------------------*/

static const TXT_type vrf_string = "vector-ref";

static NIL_type vector_ref_native(VEC_type Vector,
                                  CNT_type Continuation)
  { EXP_type value;
    LNG_type raw_number;
    NBR_type number;
    UNS_type size;
    VEC_type vector;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      vrf_string);
    vector = Vector[1];
    if (!is_VEC(vector))
      Main_Error_Text(NAV_error_string,
                      vrf_string);
    size = size_VEC(vector);
    number = Vector[2];
    if (!is_NBR(number))
      Main_Error_Text(NAN_error_string,
                      vrf_string);
    raw_number = number->lng;
    if (raw_number < 0)
      Main_Error_Text(NNN_error_string,
                      vrf_string);
    if (raw_number >= size)
      Main_Error_Text(IOR_error_string,
                      vrf_string);
    value = vector[raw_number + 1];
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_vector_ref_native(NIL_type)
  { native_define(vrf_string,
                  vector_ref_native); }

/*------------------------------ vector-set! ---------------------------------*/

static const TXT_type vst_string = "vector-set!";

static NIL_type vector_set_native(VEC_type Vector,
                                  CNT_type Continuation)
  { EXP_type value;
    LNG_type raw_number;
    NBR_type number;
    UNS_type size;
    VEC_type vector;
    size = size_VEC(Vector);
    if (size != 3)
      Main_Error_Text(EX3_error_string,
                      vst_string);
    vector = Vector[1];
    if (!is_VEC(vector))
      Main_Error_Text(NAV_error_string,
                      vst_string);
    size = size_VEC(vector);
    number = Vector[2];
    if (!is_NBR(number))
      Main_Error_Text(NAN_error_string,
                      vst_string);
    raw_number = number->lng;
    if (raw_number < 0)
      Main_Error_Text(NNN_error_string,
                      vst_string);
    if (raw_number >= size)
      Main_Error_Text(IOR_error_string,
                      vst_string);
    value =  Vector[3];
    vector[raw_number + 1] = value;
    Evaluate_Continue(vector,
                      Continuation); }

static NIL_type initialize_vector_set_native(NIL_type)
  { native_define(vst_string,
                  vector_set_native); }

/*------------------------------ vector --------------------------------------*/

static const TXT_type vec_string = "vector";

static NIL_type vector_native(VEC_type Vector,
                              CNT_type Continuation)
  { Evaluate_Continue(Vector,
                      Continuation); }

static NIL_type initialize_vector_native(NIL_type)
  { native_define(vec_string,
                  vector_native); }

/*------------------------------ vector-length -------------------------------*/

static const TXT_type vcl_string = "vector-length";

static NIL_type vector_length_native(VEC_type Vector,
                                     CNT_type Continuation)
  { EXP_type value;
    UNS_type size;
    VEC_type vector;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                      vcl_string);
    vector = Vector[1];
    if (!is_VEC(vector))
      Main_Error_Text(NAV_error_string,
                      vcl_string);
    size = size_VEC(vector);
    value = make_NBR(size);
    Evaluate_Continue(value,
                      Continuation); }

static NIL_type initialize_vector_length_native(NIL_type)
  { native_define(vcl_string,
                  vector_length_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------ control -------------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ apply ---------------------------------------*/

static const TXT_type apl_string = "apply";

static NIL_type apply_native(VEC_type Vector,
                             CNT_type Continuation)
  { EXP_type arguments,
             procedure;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      apl_string);
    procedure = Vector[1];
    arguments = Vector[2];
    Evaluate_Apply(procedure,
                   arguments,
                   Continuation); }

static NIL_type initialize_apply_native(NIL_type)
  { native_define(apl_string,
                  apply_native); }

/*------------------------------ map -----------------------------------------*/

static const TXT_type map_string = "map";

static CFN_type Continue_map;

typedef struct mAP * mAP_type;
typedef struct mAP { CEL_type hdr;
                     CFN_type cfn;
                     CNT_type cnt;
                     EXP_type prc;
                     EXP_type arg;
                     PAI_type lst; } mAP;

static const UNS_type mAP_size = chunk_size(mAP);

static NIL_type map(CNT_type Continuation,
                    EXP_type Procedure,
                    PAI_type Arguments,
                    EXP_type List)
  { mAP_type map_thread;
    CNT_type continuation;
    EXP_type value;
    PAI_type arguments,
             pair;
    value     = Arguments->car;
    arguments = Arguments->cdr;
    continuation = make_CNT(Continue_map,
                            Continuation,
                            mAP_size);
    map_thread = (mAP_type)continuation;
    map_thread->prc = Procedure;
    map_thread->arg = arguments;
    map_thread->lst = List;
    pair = make_PAI(value,
                    Main_Empty_Pair);
    Evaluate_Apply(Procedure,
                   pair,
                   continuation); }

static NIL_type continue_map(EXP_type Value,
                             CNT_type Continuation)
  { mAP_type map_thread;
    CNT_type continuation;
    EXP_type procedure;
    PAI_type arguments,
             list;
    TAG_type tag;
    map_thread = (mAP_type)Continuation;
    continuation = map_thread->cnt;
    arguments    = map_thread->arg;
    list         = map_thread->lst;
    tag = Tag_of(arguments);
    list = make_PAI(Value,
                    list);
    switch (tag)
      { case NUL_tag:
          list = Main_Reverse(list);
          Evaluate_Continue(list,
                            continuation);
        case PAI_tag:
          procedure = map_thread->prc;
          map(continuation,
              procedure,
              arguments,
              list); }
    Main_Error_Text(NAL_error_string,
                    map_string); }

/*----------------------------------------------------------------------------*/

static NIL_type map_native(VEC_type Vector,
                           CNT_type Continuation)
  { EXP_type arguments,
             procedure;
    TAG_type tag;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 2)
      Main_Error_Text(EX2_error_string,
                      map_string);
    arguments = Vector[2];
    tag = Tag_of(arguments);
    switch (tag)
      { case NUL_tag:
          Evaluate_Continue(Main_Empty_Pair,
                            Continuation);
        case PAI_tag:
          procedure = Vector[1];
          map(Continuation,
              procedure,
              arguments,
              Main_Empty_Pair); }
    Main_Error_Text(NAL_error_string,
                          map_string); }

static NIL_type initialize_map_native(NIL_type)
  { Continue_map = make_CFN(continue_map);
    native_define(map_string,
                  map_native); }

/*------------------------------ eval ----------------------------------------*/

static const TXT_type evl_string = "eval";

static NIL_type eval_native(VEC_type Vector,
                            CNT_type Continuation)
  { EXP_type expression;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                      evl_string);
    expression = Vector[1];
    Evaluate_Evaluate(expression,
                      Continuation); }

static NIL_type initialize_eval_native(NIL_type)
  { native_define(evl_string,
                  eval_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------- input --------------------------------------*/
/*----------------------------------------------------------------------------*/

/*-------------------------------- read --------------------------------------*/

static const TXT_type rdd_string = "read";

static NIL_type read_native(VEC_type Vector,
                            CNT_type Continuation)
  { EXP_type expression;
    STR_type name;
    TXT_type input,
             text;
    UNS_type size;
    size = size_VEC(Vector);
    if (size > 1)
      Main_Error_Text(AM1_error_string,
                      rdd_string);
    if (size == 0)
      input = Main_Read();
    else
      { name = Vector[1];
        if (!is_STR(name))
          Main_Error_Text(NAS_error_string,
                          rdd_string);
        text = name->txt;
        input = Main_Load(text); }
    expression = Read_Parse(input);
    Evaluate_Continue(expression,
                      Continuation); }
   
static NIL_type initialize_read_native(NIL_type)
  { native_define(rdd_string,
                  read_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------ output --------------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------ display -------------------------------------*/

static const TXT_type dis_string = "display";

static NIL_type display_native(VEC_type Vector,
                               CNT_type Continuation)
  { EXP_type value;
    UNS_type size;
    size = size_VEC(Vector);
    if (size != 1)
      Main_Error_Text(EX1_error_string,
                      dis_string);
    value = Vector[1];
    Print_Display(value);
    Evaluate_Continue(Main_Unspecified,
                      Continuation); }

static NIL_type initialize_display_native(NIL_type)
  { native_define(dis_string,
                  display_native); }

/*------------------------------ newline -------------------------------------*/

static const TXT_type nwl_string = "newline";

static NIL_type newline_native(VEC_type Vector,
                               CNT_type Continuation)
  { UNS_type size;
    size = size_VEC(Vector);
    if (size != 0)
      Main_Error_Text(EX0_error_string,
                      nwl_string);
    Print_Newline();
    Evaluate_Continue(Main_Unspecified,
                      Continuation); }

static NIL_type initialize_newline_native(NIL_type)
  { native_define(nwl_string,
                  newline_native); }

/*----------------------------------------------------------------------------*/
/*------------------------------ service -------------------------------------*/
/*----------------------------------------------------------------------------*/

/*------------------------------- clock --------------------------------------*/

static const TXT_type clk_string = "clock";

static NIL_type clock_native(VEC_type Vector,
                             CNT_type Continuation)
  { FLO_type raw_real;
    REA_type real;
    UNS_type size,
             ticks;
    size = size_VEC(Vector);
    if (size != 0)
      Main_Error_Text(EX0_error_string,
                      clk_string);
    ticks = clock();
    raw_real = (FLO_type)ticks / CLOCKS_PER_SEC;
    real = make_REA(raw_real);
    Evaluate_Continue(real,
                      Continuation); }

static NIL_type initialize_clock_native(NIL_type)
  { native_define(clk_string,
                  clock_native); }

/*--------------------------------- exit -------------------------------------*/

static const TXT_type exi_string = "exit";

static NIL_type exit_native(VEC_type Vector,
                            CNT_type Continuation)
  { UNS_type size;
    size = size_VEC(Vector);
    if (size != 0)
      Main_Error_Text(EX0_error_string,
                      exi_string);
    Main_Terminate(); }

static NIL_type initialize_exit_native(NIL_type)
  { native_define(exi_string,
                  exit_native); }
                  
/*------------------------------ public functions ----------------------------*/

NIL_type Native_Initialize(NIL_type)
  { initialize_add_native();
    initialize_apply_native();
    initialize_assoc_native();
    initialize_car_native();
    initialize_cdr_native();
    initialize_circularity_level();
    initialize_clock_native();
    initialize_cons_native();
    initialize_display_native();
    initialize_divide_native();
    initialize_equal_native();
    initialize_equivalent_native();
    initialize_eval_native();
    initialize_exit_native();
    initialize_greater_native();
    initialize_greater_or_equal_native();
    initialize_is_null_native();
    initialize_is_pair_native();
    initialize_is_symbol_native();
    initialize_is_vector_native();
    initialize_less_native();
    initialize_less_or_equal_native();
    initialize_make_vector_native();
    initialize_map_native();
    initialize_multiply_native();
    initialize_newline_native();
    initialize_not_native();
    initialize_quotient_native();
    initialize_read_native();
    initialize_remainder_native();
    initialize_set_car_native();
    initialize_set_cdr_native();
    initialize_sqrt_native();
    initialize_subtract_native();
    initialize_vector_length_native();
    initialize_vector_native();
    initialize_vector_ref_native();
    initialize_vector_set_native(); }