                     /*-----------------------------------*/
                     /*             >>>Slip<<<            */
                     /*            Theo D'Hondt           */
                     /*     VUB Software Languages Lab    */
                     /*             (c) 2010              */
                     /*-----------------------------------*/
                     /*version 3: factored out environment*/
                     /*-----------------------------------*/
                     /*               print               */
                     /*-----------------------------------*/

#include <stdio.h>

#include "SlipMain.h"

#include "SlipPrint.h"

/*------------------------------ private constants ---------------------------*/

static const TXT_type ILT_error_string = "illegal expression type";
static const TXT_type IXT_error_string = "invalid expression type";

static const TXT_type   Character_text = "%c";
static const TXT_type  Empty_list_text = "()";
static const TXT_type       False_text = "#f";
static const TXT_type     Leftpar_text = "(";
static const TXT_type     Leftvec_text = "[";
static const TXT_type      Native_text = "<native %s>";
static const TXT_type     Newline_text = "\n";
static const TXT_type      Number_text = "%ld";
static const TXT_type      Period_text = " . ";
static const TXT_type   Procedure_text = "<procedure %s>";
static const TXT_type        Real_text = "%#.10g";
static const TXT_type    Rightpar_text = ")";
static const TXT_type    Rightvec_text = "]";
static const TXT_type       Space_text = " ";
static const TXT_type        True_text = "#t";
static const TXT_type Unspecified_text = "<unspecified>";

/*------------------------------ private prototypes --------------------------*/

static NIL_type print_expression(EXP_type);

/*------------------------------ private functions ---------------------------*/

static NIL_type print_character(CHA_type Character)
  { snprintf(Main_Buffer,
             Main_Buffer_Size,
             Character_text,
             Character->chr);
    Main_Print(Main_Buffer); }

static NIL_type print_false(NIL_type)
  { Main_Print(False_text); }

static NIL_type print_native(NAT_type Native)
  { TXT_type name = Native->nam;
    snprintf(Main_Buffer,
             Main_Buffer_Size,
             Native_text,
             name);
    Main_Print(Main_Buffer); }

static NIL_type print_newline(NIL_type)
  { Main_Print(Newline_text); }

static NIL_type print_null(NIL_type)
  { Main_Print(Empty_list_text); }

static NIL_type print_number(NBR_type Number)
  { LNG_type raw_number;
    raw_number = Number->lng;
    snprintf(Main_Buffer,
             Main_Buffer_Size,
             Number_text,
             raw_number);
    Main_Print(Main_Buffer); }

static NIL_type print_pair(PAI_type Pair)
  { EXP_type residue,
             value;
    PAI_type list;
    TAG_type tag;
    Main_Print(Leftpar_text);
    for (list = Pair;
         ;
         )
      { value = list->car;
        residue = list->cdr;
        print_expression(value);
        tag = Tag_of(residue);
        switch (tag)
          { case NUL_tag:
              Main_Print(Rightpar_text);
              return;
            case PAI_tag:
              Main_Print(Space_text);
              list = residue;
              break;
            default:
              Main_Print(Period_text);
              print_expression(residue);
              Main_Print(Rightpar_text);
              return; }}}

static NIL_type print_procedure(PRC_type Procedure)
  { SYM_type name;
    TXT_type text;
    name = Procedure->nam;
    text = name->txt;
    snprintf(Main_Buffer,
             Main_Buffer_Size,
             Procedure_text,
             text);
    Main_Print(Main_Buffer); }

static NIL_type print_real(REA_type Real)
  { snprintf(Main_Buffer,
             Main_Buffer_Size,
             Real_text,
             Real->flo);
    Main_Print(Main_Buffer); }

static NIL_type print_string(STR_type String)
  { TXT_type text;
    text = String->txt;
    Main_Print(text); }

static NIL_type print_symbol(SYM_type Symbol)
  { TXT_type text;
    text = Symbol->txt;
    Main_Print(text); }

static NIL_type print_true(NIL_type)
  { Main_Print(True_text); }

static NIL_type print_unspecified(NIL_type)
  { Main_Print(Unspecified_text); }

static NIL_type print_vector(VEC_type Vector)
  { EXP_type value;
    UNS_type index,
             size;
    size = size_VEC(Vector);
    Main_Print(Leftvec_text);
    for (index = 1;
         index <= size;
         index++)
      { value = Vector[index];
        print_expression(value);
        if (index < size)
          Main_Print(Space_text); }
    Main_Print(Rightvec_text); }

static NIL_type print_expression(EXP_type Value)
  { TAG_type tag;
    tag = Tag_of(Value);
    switch (tag)
      { case CHA_tag:
          print_character(Value);
          return;
        case FLS_tag:
          print_false();
          return;
        case NAT_tag:
          print_native(Value);
          return;
        case NBR_tag:
          print_number(Value);
          return;
        case NUL_tag:
          print_null();
          return;
        case PAI_tag:
          print_pair(Value);
          return;
        case PRC_tag:
          print_procedure(Value);
          return;
        case REA_tag:
          print_real(Value);
          return;
        case STR_tag:
          print_string(Value);
          return;
        case SYM_tag:
          print_symbol(Value);
          return;
        case TRU_tag:
          print_true();
          return;
        case USP_tag:
          print_unspecified();
          return;
        case VEC_tag:
          print_vector(Value);
          return; 
        case CFN_tag:
        case CNT_tag:
        case ENV_tag:
          Main_Error_Tag(ILT_error_string,
                         tag); }
    Main_Error_Tag(IXT_error_string,
                   tag); }

/*------------------------------ public functions ----------------------------*/

NIL_type Print_Display(EXP_type Value)
  { print_expression(Value); }

NIL_type Print_Initialize(NIL_type)
  { return; }

NIL_type Print_Newline(NIL_type)
  { print_newline(); }

NIL_type Print_Print(EXP_type Value)
  { print_expression(Value); }